/*global require, module */
module.exports = function (grunt) {
  "use strict";

  var LIVERELOAD_PORT, lrSnippet, livereloadMiddleware;

  // This is the default port that livereload listens on;
  // change it if you configure livereload to use another port.
  LIVERELOAD_PORT = 35729;
  // lrSnippet is just a function.
  // It's a piece of Connect middleware that injects
  // a script into the static served html.
  lrSnippet = require('connect-livereload')({port: LIVERELOAD_PORT});
  // All the middleware necessary to serve static files.
  livereloadMiddleware = function (connect, options) {
    return [
      // Inject a livereloading script into static files.
      lrSnippet,
      // Serve static files.
      connect.static(options.base),
      // Make empty directories browsable.
      connect.directory(options.base)
    ];
  };

  grunt.loadNpmTasks("grunt-jslint");
  grunt.loadNpmTasks('grunt-contrib-watch');
  grunt.loadNpmTasks('grunt-contrib-qunit');
  grunt.loadNpmTasks('grunt-contrib-connect');
  grunt.loadNpmTasks('grunt-contrib-copy');
  grunt.loadNpmTasks('grunt-contrib-less');
  grunt.loadNpmTasks('grunt-open');

  grunt.registerMultiTask('files_to_json', 'Simple plugin to create a list of files', function () {
    // Iterate over all specified file groups.
    this.files.forEach(function (f) {
      var output = '[\n';
      output += f.src.filter(function (filepath) {
        // Warn on and remove invalid source files (if nonull was set).
        if (!grunt.file.exists(filepath)) {
          grunt.log.warn('Source file "' + filepath + '" not found.');
          return false;
        }
        return true;
      }).map(function (filepath) {
        return '  \"' + filepath + '\"';
      }).join(grunt.util.normalizelf(',\n'));
      output += '\n]';

      // Write the destination file.
      grunt.file.write(f.dest, output);
    });
  });

  grunt.initConfig({
    pkg: grunt.file.readJSON('package.json'),

    jslint: {
      config: {
        src: ['package.json', 'Gruntfile.js'],
        directives: {
          maxlen: 100,
          indent: 2,
          maxerr: 3,
          predef: [
            'module'
          ]
        }
      },
      test: {
        src: ['test/jsonform_test.js', 'test/schema_test.js'],
        directives: {
          maxlen: 79,
          indent: 2,
          maxerr: 3,
          unparam: true,
          predef: [
            'window',
            'document',
            'QUnit',
            'renderJS',
            'rJS',
            'sinon',
            'RSVP'
          ]
        }
      },
      client: {
        src: ['jsonform.gadget.js',
              'jsonform/gadget_json_generated_form_child.js',
              'demo/gadget_demo_json_schema_form.js'],
        directives: {
          maxlen: 79,
          indent: 2,
          maxerr: 3,
          unparam: true,
          predef: [
            'RSVP',
            'window',
            'document'
          ]
        }
      }
    },
    less: {
      production: {
        files: {
          'gadget_erp5_nojqm.css': 'erp5css.less'
        }
      }
    },

    copy: {
      dependencies: {
        files: [{
          src: 'node_modules/rsvp/dist/rsvp-2.0.4.js',
          dest: "rsvp.js"
        }, {
          src: 'node_modules/renderjs/dist/renderjs-latest.js',
          dest: "renderjs.js"
        }, {
          src: 'node_modules/jio/dist/jio-latest.js',
          dest: "jio.js"
        }, {
          src: 'node_modules/handlebars/dist/handlebars.js',
          dest: "handlebars.js"
        }, {
          src: 'node_modules/font-awesome/fonts/fontawesome-webfont.ttf',
          dest: "font-awesome/font-awesome-webfont.ttf"
        }, {
          src: 'node_modules/font-awesome/fonts/fontawesome-webfont.woff',
          dest: "font-awesome/font-awesome-webfont.woff"
        }, {
          src: 'node_modules/font-awesome/fonts/fontawesome-webfont.woff2',
          dest: "font-awesome/font-awesome-webfont.woff2"
        }, {
          src: 'node_modules/font-awesome/fonts/fontawesome-webfont.svg',
          dest: "font-awesome/font-awesome-webfont.svg"
        }, {
          src: 'node_modules/font-awesome/fonts/fontawesome-webfont.eot',
          dest: "font-awesome/font-awesome-webfont.eot"
        }]
      }
    },

    watch: {
      src: {
        files: [
          '<%= jslint.client.src %>',
          '<%= jslint.config.src %>',
          '<%= jslint.test.src %>',
          ['test/*.html', 'test/*.js']
        ],
        tasks: ['default'],
        options: {
          livereload: LIVERELOAD_PORT
        }
      }
    },

    qunit: {
      all: ['test/index.html']
    },

    files_to_json: {
      files: {
        src: 'node_modules/json-schema-test-suite/tests/draft7/*.json',
        dest: 'test_index.json'
      }
    },
    connect: {
      client: {
        options: {
          port: 9000,
          base: '.',
          directory: '.',
          middleware: livereloadMiddleware
        }
      }
    },

    open: {
      all: {
        // Gets the port from the connect configuration
        path: 'http://localhost:<%= connect.client.options.port%>/test/'
      }
    }
  });

  grunt.registerTask('default', ['all']);
  grunt.registerTask('all', ['less', 'lint']);
  grunt.registerTask('lint', ['jslint']);
  grunt.registerTask('test', ['copy', 'qunit']);
  grunt.registerTask('server', ['connect:client', 'files_to_json', 'watch']);

};
