/*jslint nomen: true, maxlen: 200, indent: 2*/
/*global rJS, console, window, document, RSVP, jIO*/

(function (window, rJS, jIO) {
  "use strict";

  function downloadJSON(url) {
    return RSVP.Queue()
      .push(function () {
        return jIO.util.ajax({
          url: url,
          dataType: "json"
        });
      })
      .push(undefined, function (error) {
        console.log(error);
        throw error;
      })
      .push(function (evt) {
        return evt.target.response;
      });
  }

  function render_form(gadget, scope, schema, value) {
    return gadget.getDeclaredGadget(scope)
      .push(function (g) {
        return g.render({
          schema: schema,
          value: value
        });
      });
  }

  function render_schema_selection_form(gadget, url) {
    if (gadget.props.schemas.indexOf(url) < 0) {
      gadget.props.schemas.push(url);
    }
    return render_form(gadget, "schema", {
      default: url,
      enum: gadget.props.schemas
    });
  }

  function render_document_selection_form(gadget) {
    return render_form(gadget, "document", {
      enum: gadget.props.documents
    }, gadget.props.documents[0]);
  }

  function updateTextContent(g) {
    return g.getDeclaredGadget("form_view")
      .push(function (form_view) {
        return form_view.getContent();
      })
      .push(function (ret) {
        var valid = g.state.valid,
          changed = g.state.changed;
        if (typeof valid === "boolean") {
          if (valid) {
            valid = "form filed valid";
          } else {
            valid = "form filed invalid";
          }
        } else {
          valid = "valid status unknown";
        }
        if (changed) {
          changed = 'changed\n';
        } else {
          changed = '';
        }
        document.getElementById("json_document_content").textContent =
          valid + '\n' +
          changed +
          JSON.stringify(ret, null, 2);
      });
  }

  rJS(window)
    .ready(function (g) {
      g.props = {};
      return RSVP.Queue()
        .push(function () {
          return RSVP.all([
            downloadJSON("schemas.json"),
            downloadJSON("docs.json")
          ]);
        })
        .push(function (arr) {
          var schemas = arr[0],
            docs = arr[1];
          g.props.schemas = docs.filter(function (v) {
            var suffix = "_schema.json";
            return v.indexOf(suffix, v.length - suffix.length) !== -1;
          });
          g.props.schemas = g.props.schemas.concat([
            "http://json-schema.org/draft-07/schema",
            "http://json-schema.org/draft-06/schema",
            "http://json-schema.org/draft-04/schema"
          ]);
          g.props.schemas = g.props.schemas.concat(schemas);
          g.props.schemas = ["form generated by schema"].concat(g.props.schemas);
          g.props.documents = docs.concat(schemas);
          g.props.documents = ["open document"].concat(g.props.documents);
        })
        .push(function () {
          return RSVP.all([
            render_form(g, "schema", {
              type: "string",
              enum: g.props.schemas
            }, g.props.schemas[0]),
            render_document_selection_form(g)
          ]);
        });
    })
    .allowPublicAcquisition("notifyValid", function (arr, scope) {
      if (scope === "form_view") {
        this.changeState({
          valid: true
        });
      }
    })
    .allowPublicAcquisition("notifyInvalid", function (arr, scope) {
      if (scope === "form_view") {
        this.changeState({
          valid: false
        });
      }
    })
    .allowPublicAcquisition("notifyChange", function (arr, scope) {
      var gadget = this;
      if (scope === "schema") {
        return this.getDeclaredGadget("schema")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            if (ret !== gadget.props.schemas[0] &&
                ret !== gadget.state.schema_url) {
              return gadget.changeState({
                schema_url: ret,
                json_document: gadget.state.json_document
              });
            }
          });
      }
      if (scope === "document") {
        return gadget.getDeclaredGadget("document")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            if (ret !== gadget.props.documents[0]) {
              return downloadJSON(ret);
            }
            return gadget.state.json_document;
          })
          .push(function (json_document) {
            var schema_url = gadget.state.schema_url,
              tasks = [];
            if (json_document.$schema) {
              if (json_document.$schema !== schema_url) {
                tasks.push(render_schema_selection_form(gadget, json_document.$schema));
              }
              schema_url = json_document.$schema;
            }
            tasks.push(gadget.changeState({
              schema_url: schema_url,
              json_document: json_document
            }));
            return RSVP.all(tasks);
          });
      }
      if (scope === "form_view") {
        return gadget.changeState({
          changed: true
        })
          .push(function () {
            return updateTextContent(gadget);
          });
      }
    })
    .onStateChange(function (modification) {
      var g = this,
        queue = RSVP.Queue();
      if (modification.hasOwnProperty('json_document') ||
          modification.hasOwnProperty('schema') ||
          modification.hasOwnProperty('schema_url')) {
        queue
          .push(function () {
            return g.getDeclaredGadget("form_view");
          })
          .push(function (form_view) {
            return form_view.render({
              value: g.state.json_document,
              schema: g.state.schema,
              schema_url: g.state.schema_url
            });
          })
          .push(undefined, function (error) {
            console.log(error);
          })
          .push(function () {
            return updateTextContent(g);
          });
      }
      if (modification.hasOwnProperty('valid')) {
        queue
          .push(function () {
            return updateTextContent(g);
          });
      }
      return queue;
    });

}(window, rJS, jIO));