/*jslint nomen: true, maxlen: 200, indent: 2*/
/*global rJS, console, window, document, RSVP, jIO*/

(function (window, rJS, jIO) {
  "use strict";

  function replaceAll(str, searchStr, replaceStr) {
    searchStr = searchStr.replace(/[\-\/\\\^$*+?.()|\[\]{}]/g, '\\$&');
    return str.replace(new RegExp(searchStr, 'gi'), replaceStr);
  }

  function downloadJSON(url) {
    return RSVP.Queue()
      .push(function () {
        return jIO.util.ajax({
          url: url,
          dataType: "json"
        });
      })
      .push(undefined, function (error) {
        console.log(error);
        throw error;
      })
      .push(function (evt) {
        return evt.target.response;
      });
  }

  function render_form(gadget, scope, schema, value) {
    return gadget.getDeclaredGadget(scope)
      .push(function (g) {
        return g.render({
          schema: schema,
          value: value
        });
      });
  }

  function render_document_selection_form(gadget, schema) {
    var documents = gadget.props.documents[schema] || [""];
    return render_form(gadget, "document", {
      default: documents[0],
      enum: documents
    });
  }

  function updateTextContent(g) {
    return g.getDeclaredGadget("form_view")
      .push(function (form_view) {
        return form_view.getContent();
      })
      .push(function (ret) {
        var valid = g.state.valid,
          changed = g.state.changed;
        if (typeof valid === "boolean") {
          if (valid) {
            valid = "form filed valid";
          } else {
            valid = "form filed invalid";
          }
        } else {
          valid = "valid status unknown";
        }
        if (changed) {
          changed = 'changed\n';
        } else {
          changed = '';
        }
        document.getElementById("json_document_content").textContent =
          valid + '\n' +
          changed +
          JSON.stringify(ret, null, 2);
      });
  }

  rJS(window)
    .ready(function (g) {
      var url_list = [];
      g.props = {};
      g.props.schemas = [];
      g.props.documents = [];
      g.props.test_data = {};
      return RSVP.Queue()
        .push(function () {
          return downloadJSON("../test_index.json");
        })
        .push(function (list) {
          var i,
            url,
            tasks = [];
          for (i = 0; i < list.length; i += 1) {
            url = '../' + list[i];
            tasks.push(downloadJSON(url));
            url_list.push(url);
          }
          return RSVP.all(tasks);
        })
        .push(function (list) {
          var i,
            k,
            z,
            m,
            schema,
            t;
          g.props.schemas.push("");
          for (i = 0; i < list.length; i += 1) {
            for (k = 0; k < list[i].length; k += 1) {
              m = list[i][k];
              g.props.schemas.push(m.description);
              g.props.documents[m.description] = [];
              schema = JSON.parse(replaceAll(JSON.stringify(m.schema), "http://localhost:1234/",
                "http://127.0.0.1:9000/node_modules/json-schema-test-suite/remotes/"));
              g.props.documents[m.description].push('undefined');
              g.props.test_data[m.description + ': undefined'] = {
                schema: schema,
                schema_url: url_list[i] + '#/' + k + '/schema',
                data: undefined
              };
              for (z = 0; z < m.tests.length; z += 1) {
                t = m.tests[z];
                g.props.documents[m.description].push(t.description);
                g.props.test_data[m.description + ': ' + t.description] = {
                  schema: schema,
                  schema_url: url_list[i] + '#/' + k + '/schema',
                  data: t.data,
                  status: t.valid
                };
              }
            }
          }
        })
        .push(function () {
          return RSVP.all([
            render_form(g, "schema", {
              type: "string",
              enum: g.props.schemas
            }, g.props.schemas[0])
          ]);
        });
    })
    .allowPublicAcquisition("notifyValid", function (arr, scope) {
      if (scope === "form_view") {
        this.changeState({
          valid: true
        });
      }
    })
    .allowPublicAcquisition("notifyInvalid", function (arr, scope) {
      if (scope === "form_view") {
        this.changeState({
          valid: false
        });
      }
    })
    .allowPublicAcquisition("notifyChange", function (arr, scope) {
      var gadget = this;
      if (scope === "schema") {
        return this.getDeclaredGadget("schema")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            gadget.props.schema = ret;
            return render_document_selection_form(gadget, ret);
          });
      }
      if (scope === "document") {
        return gadget.getDeclaredGadget("document")
          .push(function (g) {
            return g.getContent();
          })
          .push(function (ret) {
            gadget.props.valid = null;
            var test = gadget.props.schema + ': ' + ret;
            test = gadget.props.test_data[test];
            if (test) {
              console.log(test.schema);
              console.log(test.data);
              return gadget.changeState({
                changed: false,
                schema: test.schema,
                json_document: test.data
              });
            }
          });
      }
      if (scope === "form_view") {
        return gadget.changeState({
          changed: true
        })
          .push(function () {
            return updateTextContent(gadget);
          });
      }
    })
    .onStateChange(function (modification) {
      var g = this,
        queue = RSVP.Queue();
      if (modification.hasOwnProperty('json_document') ||
          modification.hasOwnProperty('schema') ||
          modification.hasOwnProperty('schema_url')) {
        queue
          .push(function () {
            return g.getDeclaredGadget("form_view");
          })
          .push(function (form_view) {
            return form_view.render({
              value: g.state.json_document,
              schema: g.state.schema,
              schema_url: g.state.schema_url
            });
          })
          .push(undefined, function (error) {
            console.log(error);
          })
          .push(function () {
            return updateTextContent(g);
          });
      }
      if (modification.hasOwnProperty('valid')) {
        queue
          .push(function () {
            return updateTextContent(g);
          });
      }
      return queue;
    });

}(window, rJS, jIO));